@@ -3,6 +3,7 @@
 use std::{
     fmt::{self, Debug, Display},
     io,
+    iter::Iterator,
     net::{Ipv4Addr, SocketAddr},
     sync::{
         atomic::{AtomicUsize, Ordering},
@@ -11,11 +12,13 @@ use std::{
     time::{Duration, Instant},
 };
 
+use arc_swap::ArcSwap;
 use byte_string::ByteStr;
 use futures::future;
-use log::{debug, info, trace};
+use log::{debug, error, info, trace, warn};
 use shadowsocks::{
     config::Mode,
+    plugin::{Plugin, PluginMode},
     relay::{
         socks5::Address,
         tcprelay::proxy_stream::ProxyClientStream,
@@ -23,6 +26,7 @@ use shadowsocks::{
     },
     ServerConfig,
 };
+use spin::Mutex as SpinMutex;
 use tokio::{
     io::{AsyncBufReadExt, AsyncWriteExt, BufReader},
     task::JoinHandle,
@@ -57,6 +61,8 @@ pub struct PingBalancerBuilder {
     servers: Vec<Arc<ServerIdent>>,
     context: Arc<ServiceContext>,
     mode: Mode,
+    max_server_rtt: Duration,
+    check_interval: Duration,
 }
 
 impl PingBalancerBuilder {
@@ -65,40 +71,109 @@ impl PingBalancerBuilder {
             servers: Vec::new(),
             context,
             mode,
+            max_server_rtt: Duration::from_secs(DEFAULT_CHECK_TIMEOUT_SEC),
+            check_interval: Duration::from_secs(DEFAULT_CHECK_INTERVAL_SEC),
         }
     }
 
-    pub fn add_server(&mut self, server: ServerIdent) {
-        self.servers.push(Arc::new(server));
+    pub fn add_server(&mut self, server: ServerConfig) {
+        let ident = ServerIdent::new(server, self.max_server_rtt);
+        self.servers.push(Arc::new(ident));
     }
 
-    pub async fn build(self) -> PingBalancer {
+    pub fn max_server_rtt(&mut self, rtt: Duration) {
+        self.max_server_rtt = rtt;
+    }
+
+    pub fn check_interval(&mut self, intv: Duration) {
+        self.check_interval = intv;
+    }
+
+    fn find_best_idx(servers: &[Arc<ServerIdent>], mode: Mode) -> (usize, usize) {
+        let mut best_tcp_idx = 0;
+        let mut best_udp_idx = 0;
+
+        if mode.enable_tcp() {
+            let mut found_tcp_idx = false;
+            for (idx, server) in servers.iter().enumerate() {
+                if PingBalancerContext::check_server_tcp_enabled(server.server_config()) {
+                    best_tcp_idx = idx;
+                    found_tcp_idx = true;
+                    break;
+                }
+            }
+
+            if !found_tcp_idx {
+                warn!(
+                    "no valid TCP server serving for TCP clients, consider disable TCP with \"mode\": \"udp_only\", currently chose {}",
+                    ServerConfigFormatter::new(servers[best_tcp_idx].server_config())
+                );
+            } else {
+                trace!(
+                    "init chose TCP server {}",
+                    ServerConfigFormatter::new(servers[best_tcp_idx].server_config())
+                );
+            }
+        }
+
+        if mode.enable_udp() {
+            let mut found_udp_idx = false;
+            for (idx, server) in servers.iter().enumerate() {
+                if PingBalancerContext::check_server_udp_enabled(server.server_config()) {
+                    best_udp_idx = idx;
+                    found_udp_idx = true;
+                    break;
+                }
+            }
+
+            if !found_udp_idx {
+                warn!(
+                    "no valid UDP server serving for UDP clients, consider disable UDP with \"mode\": \"tcp_only\", currently chose {}",
+                    ServerConfigFormatter::new(servers[best_udp_idx].server_config())
+                );
+            } else {
+                trace!(
+                    "init chose UDP server {}",
+                    ServerConfigFormatter::new(servers[best_udp_idx].server_config())
+                );
+            }
+        }
+
+        (best_tcp_idx, best_udp_idx)
+    }
+
+    pub async fn build(self) -> io::Result<PingBalancer> {
         assert!(!self.servers.is_empty(), "build PingBalancer without any servers");
 
-        let balancer_context = PingBalancerContext {
-            servers: self.servers,
-            best_tcp_idx: AtomicUsize::new(0),
-            best_udp_idx: AtomicUsize::new(0),
-            context: self.context,
-            mode: self.mode,
-        };
+        let (shared_context, task_abortable) = PingBalancerContext::new(
+            self.servers,
+            self.context,
+            self.mode,
+            self.max_server_rtt,
+            self.check_interval,
+        )
+        .await?;
 
-        balancer_context.init_score().await;
-
-        let shared_context = Arc::new(balancer_context);
-
-        let abortable = {
-            let shared_context = shared_context.clone();
-            tokio::spawn(async move { shared_context.checker_task().await })
-        };
-
-        let balancer = PingBalancer {
+        Ok(PingBalancer {
             inner: Arc::new(PingBalancerInner {
-                context: shared_context,
-                abortable,
+                context: ArcSwap::new(shared_context),
+                task_abortable: SpinMutex::new(task_abortable),
             }),
-        };
-        balancer
+        })
+    }
+}
+
+struct PingBalancerContextTask {
+    checker_abortable: JoinHandle<()>,
+    plugin_abortable: Option<JoinHandle<()>>,
+}
+
+impl Drop for PingBalancerContextTask {
+    fn drop(&mut self) {
+        self.checker_abortable.abort();
+        if let Some(ref p) = self.plugin_abortable {
+            p.abort();
+        }
     }
 }
 
@@ -108,6 +183,8 @@ struct PingBalancerContext {
     best_udp_idx: AtomicUsize,
     context: Arc<ServiceContext>,
     mode: Mode,
+    max_server_rtt: Duration,
+    check_interval: Duration,
 }
 
 impl PingBalancerContext {
@@ -121,18 +198,141 @@ impl PingBalancerContext {
 }
 
 impl PingBalancerContext {
+    pub(crate) async fn new(
+        mut servers: Vec<Arc<ServerIdent>>,
+        context: Arc<ServiceContext>,
+        mode: Mode,
+        max_server_rtt: Duration,
+        check_interval: Duration,
+    ) -> io::Result<(Arc<PingBalancerContext>, PingBalancerContextTask)> {
+        let plugin_abortable = if mode.enable_tcp() {
+            // Start plugins for TCP proxies
+
+            let mut plugins = Vec::with_capacity(servers.len());
+
+            for server in &mut servers {
+                let server = Arc::get_mut(server).unwrap();
+                let svr_cfg = server.server_config_mut();
+
+                if let Some(p) = svr_cfg.plugin() {
+                    // Start Plugin Process
+                    let plugin = Plugin::start(p, svr_cfg.addr(), PluginMode::Client)?;
+                    svr_cfg.set_plugin_addr(plugin.local_addr().into());
+                    plugins.push(plugin);
+                }
+            }
+
+            if plugins.is_empty() {
+                None
+            } else {
+                // Load balancer will check all servers' score before server's actual start.
+                // So we have to ensure all plugins have been started before that.
+
+                let mut check_fut = Vec::with_capacity(plugins.len());
+
+                for plugin in &plugins {
+                    // 3 seconds is not a carefully selected value
+                    // I choose that because any values bigger will make me felt too long.
+                    check_fut.push(plugin.wait_started(Duration::from_secs(3)));
+                }
+
+                // Run all of them simutaneously
+                let _ = future::join_all(check_fut).await;
+
+                let plugin_abortable = tokio::spawn(async move {
+                    let mut vfut = Vec::with_capacity(plugins.len());
+
+                    for plugin in plugins {
+                        vfut.push(async move {
+                            match plugin.join().await {
+                                Ok(status) => {
+                                    error!("plugin exited with status: {}", status);
+                                    Ok(())
+                                }
+                                Err(err) => {
+                                    error!("plugin exited with error: {}", err);
+                                    Err(err)
+                                }
+                            }
+                        });
+                    }
+
+                    let _ = future::join_all(vfut).await;
+
+                    panic!("all plugins are exited. all connections may fail, check your configuration");
+                });
+
+                Some(plugin_abortable)
+            }
+        } else {
+            None
+        };
+
+        let (best_tcp_idx, best_udp_idx) = PingBalancerBuilder::find_best_idx(&servers, mode);
+
+        let balancer_context = PingBalancerContext {
+            servers,
+            best_tcp_idx: AtomicUsize::new(best_tcp_idx),
+            best_udp_idx: AtomicUsize::new(best_udp_idx),
+            context,
+            mode,
+            max_server_rtt,
+            check_interval,
+        };
+
+        balancer_context.init_score().await;
+
+        let shared_context = Arc::new(balancer_context);
+
+        let checker_abortable = {
+            let shared_context = shared_context.clone();
+            tokio::spawn(async move { shared_context.checker_task().await })
+        };
+
+        Ok((
+            shared_context,
+            PingBalancerContextTask {
+                checker_abortable,
+                plugin_abortable,
+            },
+        ))
+    }
+
     async fn init_score(&self) {
         assert!(!self.servers.is_empty(), "check PingBalancer without any servers");
 
-        if self.servers.len() > 1 {
-            self.check_once(true).await;
+        self.check_once(true).await;
+    }
+
+    fn check_server_tcp_enabled(svr_cfg: &ServerConfig) -> bool {
+        svr_cfg.mode().enable_tcp() && svr_cfg.weight().tcp_weight() > 0.0
+    }
+
+    fn check_server_udp_enabled(svr_cfg: &ServerConfig) -> bool {
+        svr_cfg.mode().enable_udp() && svr_cfg.weight().udp_weight() > 0.0
+    }
+
+    fn probing_required(&self) -> bool {
+        let mut tcp_count = 0;
+        let mut udp_count = 0;
+
+        for server in self.servers.iter() {
+            let svr_cfg = server.server_config();
+            if self.mode.enable_tcp() && PingBalancerContext::check_server_tcp_enabled(svr_cfg) {
+                tcp_count += 1;
+            }
+            if self.mode.enable_udp() && PingBalancerContext::check_server_udp_enabled(svr_cfg) {
+                udp_count += 1;
+            }
         }
+
+        tcp_count > 1 || udp_count > 1
     }
 
     async fn checker_task(self: Arc<Self>) {
         assert!(!self.servers.is_empty(), "check PingBalancer without any servers");
 
-        if self.servers.len() == 1 {
+        if !self.probing_required() {
             self.checker_task_dummy().await
         } else {
             self.checker_task_real().await
@@ -146,39 +346,59 @@ impl PingBalancerContext {
 
     /// Check each servers' score and update the best server's index
     async fn check_once(&self, first_run: bool) {
-        let mut vfut = match self.mode {
-            Mode::TcpAndUdp => Vec::with_capacity(self.servers.len() * 2),
-            Mode::TcpOnly | Mode::UdpOnly => Vec::with_capacity(self.servers.len()),
-        };
+        let servers = &self.servers;
 
-        for server in self.servers.iter() {
-            if self.mode.enable_tcp() {
+        let mut vfut_tcp = Vec::with_capacity(servers.len());
+        let mut vfut_udp = Vec::with_capacity(servers.len());
+
+        for server in servers.iter() {
+            let svr_cfg = server.server_config();
+
+            if self.mode.enable_tcp() && PingBalancerContext::check_server_tcp_enabled(svr_cfg) {
                 let checker = PingChecker {
                     server: server.clone(),
                     server_type: ServerType::Tcp,
                     context: self.context.clone(),
+                    max_server_rtt: self.max_server_rtt,
                 };
-                vfut.push(checker.check_update_score());
+                vfut_tcp.push(checker.check_update_score());
             }
 
-            if self.mode.enable_udp() {
+            if self.mode.enable_udp() && PingBalancerContext::check_server_udp_enabled(svr_cfg) {
                 let checker = PingChecker {
                     server: server.clone(),
                     server_type: ServerType::Udp,
                     context: self.context.clone(),
+                    max_server_rtt: self.max_server_rtt,
                 };
-                vfut.push(checker.check_update_score());
+                vfut_udp.push(checker.check_update_score());
             }
         }
 
+        let check_tcp = vfut_tcp.len() > 1;
+        let check_udp = vfut_udp.len() > 1;
+
+        if !check_tcp && !check_udp {
+            return;
+        }
+
+        let vfut = if !check_tcp {
+            vfut_udp
+        } else if !check_udp {
+            vfut_tcp
+        } else {
+            vfut_tcp.append(&mut vfut_udp);
+            vfut_tcp
+        };
+
         future::join_all(vfut).await;
 
-        if self.mode.enable_tcp() {
+        if self.mode.enable_tcp() && check_tcp {
             let old_best_idx = self.best_tcp_idx.load(Ordering::Acquire);
 
             let mut best_idx = 0;
             let mut best_score = u32::MAX;
-            for (idx, server) in self.servers.iter().enumerate() {
+            for (idx, server) in servers.iter().enumerate() {
                 let score = server.tcp_score().score();
                 if score < best_score {
                     best_idx = idx;
@@ -190,30 +410,30 @@ impl PingBalancerContext {
             if first_run {
                 info!(
                     "chose best TCP server {}",
-                    ServerConfigFormatter::new(self.servers[best_idx].server_config())
+                    ServerConfigFormatter::new(servers[best_idx].server_config())
                 );
             } else {
                 if best_idx != old_best_idx {
                     info!(
                         "switched best TCP server from {} to {}",
-                        ServerConfigFormatter::new(self.servers[old_best_idx].server_config()),
-                        ServerConfigFormatter::new(self.servers[best_idx].server_config())
+                        ServerConfigFormatter::new(servers[old_best_idx].server_config()),
+                        ServerConfigFormatter::new(servers[best_idx].server_config())
                     );
                 } else {
                     debug!(
                         "kept best TCP server {}",
-                        ServerConfigFormatter::new(self.servers[old_best_idx].server_config())
+                        ServerConfigFormatter::new(servers[old_best_idx].server_config())
                     );
                 }
             }
         }
 
-        if self.mode.enable_udp() {
+        if self.mode.enable_udp() && check_udp {
             let old_best_idx = self.best_udp_idx.load(Ordering::Acquire);
 
             let mut best_idx = 0;
             let mut best_score = u32::MAX;
-            for (idx, server) in self.servers.iter().enumerate() {
+            for (idx, server) in servers.iter().enumerate() {
                 let score = server.udp_score().score();
                 if score < best_score {
                     best_idx = idx;
@@ -225,19 +445,19 @@ impl PingBalancerContext {
             if first_run {
                 info!(
                     "chose best UDP server {}",
-                    ServerConfigFormatter::new(self.servers[best_idx].server_config())
+                    ServerConfigFormatter::new(servers[best_idx].server_config())
                 );
             } else {
                 if best_idx != old_best_idx {
                     info!(
                         "switched best UDP server from {} to {}",
-                        ServerConfigFormatter::new(self.servers[old_best_idx].server_config()),
-                        ServerConfigFormatter::new(self.servers[best_idx].server_config())
+                        ServerConfigFormatter::new(servers[old_best_idx].server_config()),
+                        ServerConfigFormatter::new(servers[best_idx].server_config())
                     );
                 } else {
                     debug!(
                         "kept best UDP server {}",
-                        ServerConfigFormatter::new(self.servers[old_best_idx].server_config())
+                        ServerConfigFormatter::new(servers[old_best_idx].server_config())
                     );
                 }
             }
@@ -246,7 +466,7 @@ impl PingBalancerContext {
 
     async fn checker_task_real(&self) {
         loop {
-            time::sleep(Duration::from_secs(DEFAULT_CHECK_INTERVAL_SEC)).await;
+            time::sleep(self.check_interval).await;
 
             // Sleep before check.
             // PingBalancer already checked once when constructing
@@ -256,13 +476,12 @@ impl PingBalancerContext {
 }
 
 struct PingBalancerInner {
-    context: Arc<PingBalancerContext>,
-    abortable: JoinHandle<()>,
+    context: ArcSwap<PingBalancerContext>,
+    task_abortable: SpinMutex<PingBalancerContextTask>,
 }
 
 impl Drop for PingBalancerInner {
     fn drop(&mut self) {
-        self.abortable.abort();
         trace!("ping balancer stopped");
     }
 }
@@ -276,36 +495,71 @@ pub struct PingBalancer {
 impl PingBalancer {
     /// Get service context
     pub fn context(&self) -> Arc<ServiceContext> {
-        self.inner.context.context.clone()
-    }
-
-    /// Get reference of the service context
-    pub fn context_ref(&self) -> &ServiceContext {
-        self.inner.context.context.as_ref()
+        let context = self.inner.context.load();
+        context.context.clone()
     }
 
     /// Pick the best TCP server
     pub fn best_tcp_server(&self) -> Arc<ServerIdent> {
-        self.inner.context.best_tcp_server()
+        let context = self.inner.context.load();
+        context.best_tcp_server()
     }
 
     /// Pick the best UDP server
     pub fn best_udp_server(&self) -> Arc<ServerIdent> {
-        self.inner.context.best_udp_server()
+        let context = self.inner.context.load();
+        context.best_udp_server()
     }
 
     /// Get the server list
-    pub fn servers(&self) -> &[Arc<ServerIdent>] {
-        &self.inner.context.servers
+    pub fn servers(&self) -> PingServerIter<'_> {
+        let context = self.inner.context.load();
+        let servers: &Vec<Arc<ServerIdent>> = unsafe { &*(&context.servers as *const _) };
+        PingServerIter {
+            context: context.clone(),
+            iter: servers.iter(),
+        }
+    }
+
+    /// Reset servers in load balancer. Designed for auto-reloading configuration file.
+    pub async fn reset_servers(&self, servers: Vec<ServerConfig>) -> io::Result<()> {
+        let old_context = self.inner.context.load();
+
+        let servers = servers
+            .into_iter()
+            .map(|s| Arc::new(ServerIdent::new(s, old_context.max_server_rtt)))
+            .collect::<Vec<Arc<ServerIdent>>>();
+
+        let (shared_context, task_abortable) = PingBalancerContext::new(
+            servers,
+            old_context.context.clone(),
+            old_context.mode,
+            old_context.max_server_rtt,
+            old_context.check_interval,
+        )
+        .await?;
+
+        {
+            // Stop the previous task and replace with the new task
+            let mut abortable = self.inner.task_abortable.lock();
+            *abortable = task_abortable;
+        }
+
+        // Replace with the new context
+        self.inner.context.store(shared_context);
+
+        Ok(())
     }
 }
 
 impl Debug for PingBalancer {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let context = self.inner.context.load();
+
         f.debug_struct("PingBalancer")
-            .field("servers", &self.inner.context.servers)
-            .field("best_tcp_idx", &self.inner.context.best_tcp_idx.load(Ordering::Relaxed))
-            .field("best_udp_idx", &self.inner.context.best_udp_idx.load(Ordering::Relaxed))
+            .field("servers", &context.servers)
+            .field("best_tcp_idx", &context.best_tcp_idx.load(Ordering::Relaxed))
+            .field("best_udp_idx", &context.best_udp_idx.load(Ordering::Relaxed))
             .finish()
     }
 }
@@ -314,6 +568,7 @@ struct PingChecker {
     server: Arc<ServerIdent>,
     server_type: ServerType,
     context: Arc<ServiceContext>,
+    max_server_rtt: Duration,
 }
 
 impl PingChecker {
@@ -453,7 +708,7 @@ impl PingChecker {
         if dns_answer.len() < 12 || &dns_answer[0..2] != b"\x12\x34" {
             use std::io::{Error, ErrorKind};
 
-            debug!("unexpected response from 8.8.8.8:53, {:?}", ByteStr::new(&dns_answer));
+            debug!("unexpected response from 8.8.8.8:53, {:?}", ByteStr::new(dns_answer));
 
             let err = Error::new(ErrorKind::InvalidData, "unexpected response from 8.8.8.8:53");
             return Err(err);
@@ -473,8 +728,7 @@ impl PingChecker {
         let start = Instant::now();
 
         // Send HTTP GET and read the first byte
-        let timeout = Duration::from_secs(DEFAULT_CHECK_TIMEOUT_SEC);
-        let res = time::timeout(timeout, self.check_request()).await;
+        let res = time::timeout(self.max_server_rtt, self.check_request()).await;
 
         let elapsed = Instant::now() - start;
         let elapsed = elapsed.as_secs() as u32 * 1000 + elapsed.subsec_millis(); // Converted to ms
@@ -542,3 +796,18 @@ impl Display for ServerConfigFormatter<'_> {
         }
     }
 }
+
+/// Server Iterator
+pub struct PingServerIter<'a> {
+    #[allow(dead_code)]
+    context: Arc<PingBalancerContext>,
+    iter: std::slice::Iter<'a, Arc<ServerIdent>>,
+}
+
+impl<'a> Iterator for PingServerIter<'a> {
+    type Item = &'a ServerIdent;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        self.iter.next().map(AsRef::as_ref)
+    }
+}
