@@ -40,9 +40,7 @@ pub async fn run(config: Config) -> io::Result<()> {
         #[cfg(target_os = "android")]
         vpn_protect_path: config.outbound_vpn_protect_path,
 
-        bind_local_addr: config.local_addr,
-
-        #[cfg(any(target_os = "linux", target_os = "android", target_os = "macos", target_os = "ios"))]
+        bind_local_addr: config.outbound_bind_addr,
         bind_interface: config.outbound_bind_interface,
 
         ..Default::default()
